/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     Angela Reunanen
 * Student ID: 9823522
 * Section:    BA
 *	
 * STARTER PROGRAM
 *
 * To start with, your job is to figure out how this program works.
 *
 * Greeting Card Designer (Ultimate version)
 *
 *  The GP142 graphics window shows some buttons and the inside
 *  or outside (cover) of a greeting card.  The idea is that you 
 *  can design what you want to card to look like by adding and moving
 *  objects: 
 *    messages, message color and position
 *    shapes,   shape color and position
 *
 *   To place a message or a greeting, you click on an appropriate button, and 
 *   then you can click on the desired position in the card.  
 
 *	 In this starter version, you can do much less.
 *   You can change the card background, place one shape on the card,
 *   and change the shape back and forth between two possible kinds.
 */

#include "gp142.h"
#include <math.h>
#include <time.h>
	/* <time.h> is here only so that one function can be called
	to seed the random number generator.
	Put it here in your program and forget about it, 
	just in case you decide later you need to use random numbers.
	*/

#define TRUE    		1
#define FALSE   		0

/*
 * The state, or status you are in determines what happens when you click 
 * the card:
 * ADD SHAPE   -- you click the card, and a new object is placed there.
 *                You get into this mode by clicking an ADD button.
 *                After doing an ADD, you are put into SELECTED mode.
 * SELECTED    -- An object is selected. In this mode, the type 
 *                rotation applies to thecurrent object.
 */
/* Hint: you may need to add statuses when you make the program more complex.*/
#define STATUS_ADD_SHAPE 1
#define STATUS_ADD_TEXT 2
#define STATUS_READY_TO_SELECT 3
#define STATUS_SELECTED 4

/* Configure the appearance of the buttons on the menu bar 
 * by setting the number of pixels of button height and width
 */
#define BUTTON_HEIGHT  	35
#define BUTTON_WIDTH 	105

/* Number of buttons and color of currently selected button. */
#define NUM_BUTTONS 	6
#define SELECTED_COLOR  RED

/* Position of the card in the window */
#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50)

/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0
#define NOFILL 1

/* Types of objects this program supports. */
#define OBJECT_NONE 0
#define OBJECT_SHAPE 1
#define OBJECT_MESSAGE 2

/* Maximum number of shapes and messages to choose from. */
#define MAX_SHAPES 7
#define FIRST_SHAPE_NUM 0
#define MAX_MESSAGES 6
#define FIRST_MESSAGE_NUM 0
#define MAX_OBJECTS 150

/* These are the kinds of objects this program supports. */
#define SHAPE_HEART 0
#define SHAPE_STAR  1
#define SHAPE_ARROW 2
#define SHAPE_ARROW2 3
#define SHAPE_CAKE 4
#define SHAPE_BALLOONS 5
#define SHAPE_TREE 6

#define HAPPY_BIRTHDAY 0
#define I_LOVE_YOU 1
#define BE_MINE 2
#define CONGRATULATIONS 3
#define MERRY_CHRISTMAS 4
#define THANK_YOU 5

/*Possible places for mouse clicks*/
#define CLICK_BUTTON 1  /* You clicked on SOME button (doesn't say which one)*/
#define CLICK_CARD 2    /* You clicked in the card area */
#define CLICK_OUTSIDE 3	/* You clicked outside the card area, not on a button*/	


/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);
void DrawButtons(int last_button);
void DrawCard (int background_color, int status, int object_count, 
			   int object_type[], int object_num[], 
			   int object_color[], int object_x[], int object_y[]);
void DrawShape(int shape_type, int color, int bottom_x, int bottom_y);
void DrawHeart (int x, int y, int color);
void DrawStar (int x, int y, int color);
void DrawArrow (int x, int y, int color);
void DrawArrow_2 (int x, int y, int color);
void DrawCake (int x, int y, int color);
void DrawBalloons (int x, int y, int color);
void DrawTree (int x, int y, int color);
void DrawMessage(int shape_type, int color, int bottom_x, int bottom_y);

/* Mouse click handling functions */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[], 
					  int object_num[], int object_color[], int object_x[],
					  int object_y[], int *current_object, int *last_button);
int ClassifyMouseClick(int mouse_x, int mouse_y);
void HandleButton(int x, int y, int *background_color, int object_color[], 
				  int *status, int object_count, int object_type[], 
				  int object_num[], int current_object, int *last_button);

/* Add a shape to the card. */
void AddShape(int x, int y, int *status, int *object_count, 
			   int *object_type, int *object_num, int *object_color, 
			   int *object_x, int *object_y, int *current_object);
void AddText(int x, int y, int *status, int *object_count, int *object_type,
			 int *object_num, int *object_color, int *object_x,
			 int *object_y, int *current_object);
void MoveObject (int x, int y, int *object_count, int object_x[], 
				 int object_y[]);
int Get_Object(int x, int y, int object_x[], int object_y[],
				int *object_count);

/*****************************************************************************

     MAIN

*****************************************************************************/

/* main() has two chief purposes: to contain the important variables and 
 *initialize them, and to contain the "event loop," which constantly looks 
 *to see if an event has happened, and then processes the event by calling 
 *an appropriate function. An example of an event is a mouse click.
 */
int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
	
	
	/*variables and their default values*/
	int background_color = YELLOW;
	int status = STATUS_ADD_SHAPE;
	
	/* Each object will have a number telling what kind of object it is, a color,
	 * and an x and y coordinate.  You'll need to make these into arrays so you
	 * can support more than one object.
	 */
	int object_count = 0;				 /* How many objects on the page? */
	int object_type[MAX_OBJECTS];		 /* Whether object is shape
										 	or message					  */
	int object_num[MAX_OBJECTS];		 /* Whichever shape or message	  
										  	i.e. Heart or "Happy Birthday"*/
										  
	int object_color[MAX_OBJECTS];		 /* Color of object               */
	int object_x[MAX_OBJECTS];			 /* Position of object            */
	int object_y[MAX_OBJECTS];
	
	int last_button = -1;		     /* Keep track of the last button clicked.
									    This would be usefull when highlighting
										the last button the user clicked  */

	int event;						 /* The most recent GP142 event		  */
	int current_object = 0;			 /* The currently selected object	  */


	time_t t; /*needed only when initializing a random number generator*/
	

	/*
	 * INITIALIZATIONS
	 */
	srand (time(&t));	    /* Seed the random number generator.			*/
	
	GP142_open();           /* Open and init the GP142 Graphics Window      */
	GP142_show_text(0);     /* Hide the text window                         */
	GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */
	
	GP142_clear();	        /* This will clear the window & paint it white. */
	GP142_animate(ANI_RUN); /* Enable periodic events                       */
	
	
	
	/*
	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this until 
	 * the user signals "quit".
	 */
	


	while (!quit) {  /*this is the famous "event loop" */
		

		/*start by redrawing everything */
		DrawBlankScreen ();
		DrawCard (background_color, status, object_count,
			      object_type, object_num, object_color, 
				  object_x, object_y);

		DrawButtons(last_button);	  /* Show the buttons...*/ 
		
		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		   appropriately. */
		switch (event) {
		case GP142_QUIT:
			quit = TRUE;           /* set flag to terminate loop           */
			break;
			
		case GP142_KBD:  /*A key was pressed */
			/*not needed for this program. Use this if user will need to 
			  hit a key. Note that scanf CANNOT be used with GP142.  If you 
			  use this, the key character value is in variable "key_hit"*/
			break;
			
		case GP142_MOUSE:
			/* A mouse click has occurred. Call the mouse handler to 
			   decide where the click was and react as needed. */
			HandleMouseClick(mouse_x, mouse_y, &background_color, &status, 
					         &object_count, object_type, 
							 object_num, object_color, 
							 object_x, object_y, &current_object, &last_button);			
			break;
			
		case GP142_PERIODIC:
			/*An event has occurred simply because some time has passed,*/
			/*not currently used in the program*/		
			break;

		default:
			
			break;
			
		}  /*end switch */
		
		
	}  /* end event loop */
	
	
	/*
	 * WRAP-UP
	 */
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  /*end main */

/*****************************************************************************

     FUNCTIONS

*****************************************************************************/
/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen (void) {
	int font_size = 14;

	GP142_rectangleXY (WHITE,
					-GP142_XMAX, -GP142_YMAX,
					GP142_XMAX, GP142_YMAX,
					FILL);

} /*end DrawBlankScreen */


/* 
 * Draws the buttons 
 */
void DrawButtons(int last_button)
{
/* To add buttons, you'll need to change the #defined constant NUM_BUTTONS
 * and add messages to this array.  Although we haven't covered 2-D arrays
 * yet, you should be able to play with this enough to get it going.
 * Look at course packet, page N-25, for a similar example.
 */
	char b_labels [NUM_BUTTONS][25] = {
			"Background color",
			"Next, please",
			"Object color",
			"Add shape",
			"Add text",
			"Select object",
	};
	
	int i, text_offset = -GP142_XMAX + 6, font_size = 12;
	int y, label_y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		y = GP142_YMAX - i * BUTTON_HEIGHT;
		
		GP142_rectangleXY(BLACK,
			-GP142_XMAX + 4,
			y - 2,
			-GP142_XMAX + BUTTON_WIDTH,
			y - BUTTON_HEIGHT - 2, 1);
		
		
		label_y = y - BUTTON_HEIGHT/2;
		
		if (i == last_button)
			GP142_printfXY(SELECTED_COLOR, text_offset, label_y, font_size, b_labels[i]);	
		else
			GP142_printfXY(BLACK, text_offset, label_y, font_size, b_labels[i]);	
	}	
}


/*
 * Render (draw) the page and all its contents.
 */
void DrawCard (int background_color, int status, int object_count, 
			   int object_type[], int object_num[], 
			   int object_color[], int object_x[], int object_y[])
{
	int font_size = 14;
	int i;
	
	/*
	 * Draw the background of the card.
	 */
	GP142_rectangleXY (background_color,
		PAGE_LLX, PAGE_LLY,
		PAGE_LLX+ PAGE_WIDTH, PAGE_LLY+ PAGE_HEIGHT,
		FILL);
	/* Draw a black border around the card. */
	GP142_rectangleXY (BLACK,
		PAGE_LLX - 3, PAGE_LLY - 3,
		PAGE_LLX + PAGE_WIDTH + 3, PAGE_LLY + PAGE_HEIGHT + 3, 4);

	/* Draw title and info */

	GP142_printfXY (BLUE, -GP142_XMAX + 8, -GP142_YMAX + 240, 18,
		"Design-A-Greeting");
	GP142_printfXY (RED, -GP142_XMAX + 17, -GP142_YMAX + 220, 13,
		"Created by:\n"
		"Angela Reunanen\n"
		"Student ID 9823522\n"
		"2-26-1999");
	
	/* 
	 * Draw an object on the card, if one currently exists.
	 */
	for (i = 0; i < object_count; i++) {
		if (object_type[i] == OBJECT_SHAPE) {
			DrawShape (object_num[i], object_color[i], object_x[i], object_y[i]);
		}
		if (object_type[i] == OBJECT_MESSAGE) {
			DrawMessage (object_num[i], object_color[i], object_x[i], object_y[i]);
		}
	}

	
	/*
	 * Write an appropriate status message below the background
	 */
	switch (status) {
	case STATUS_ADD_SHAPE:
		GP142_printfXY (GREEN, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a shape");
		break;
	case STATUS_ADD_TEXT:
		GP142_printfXY (BLUE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add some text");
		break;
	case STATUS_READY_TO_SELECT:
		GP142_printfXY (BLUE, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click near an object to select it");
		break;
	case STATUS_SELECTED:
		GP142_printfXY (RED, PAGE_LLX, PAGE_LLY-40, font_size-3,
			"Object selected. Click on card to move object or "
			"select next or object color");
		break;
	default:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer");
		break;
	} /*end switch */
	
	
} /*end DrawPage */


/*
 * Draw one shape.
 */
void DrawShape(int shape_type, int color, int bottom_x, int bottom_y) {


		switch ((shape_type - FIRST_SHAPE_NUM) % MAX_SHAPES)
		{
		case SHAPE_HEART:
			DrawHeart (bottom_x, bottom_y, color);
			break;
		case SHAPE_STAR:
			DrawStar (bottom_x, bottom_y, color);
			break;
		case SHAPE_ARROW:
			DrawArrow (bottom_x, bottom_y, color);
			break;
		case SHAPE_ARROW2:
			DrawArrow_2 (bottom_x, bottom_y, color);
			break;
		case SHAPE_CAKE:
			DrawCake (bottom_x, bottom_y, color);
			break;
		case SHAPE_BALLOONS:
			DrawBalloons (bottom_x, bottom_y, color);
			break;
		case SHAPE_TREE:
			DrawTree (bottom_x, bottom_y, color);
			break;
		default:
			/* Unknown shape selected */
			break;

		} /*end switch*/
	


}


/*
 * Draw a heart on the card. A heart will be made by two overlapping
 * circles and a triangle.
 *
 * The pulsation is caused by adding a small, random amount to the 
 * heart height each time this function is called.
 */
#define HEART_WIDTH 35
void DrawHeart (int x, int y, int color) {

	int h = (HEART_WIDTH/2) + (rand () %4);

	GP142_circleXY (color, x - h + 5, y, h);
	GP142_circleXY (color, x + h - 5, y, h);
	GP142_triangleXY (color, x- h*2 +5, y-5,
						x + h*2 -5, y-5,
						x, y - h*2 -5, FILL);

}


/*
 * Draw a star on the card.
 *
 * We make the star twinkle by selecting the number of points randomly.  Each 
 * time it is redrawn, it is slightly different than the time before, which 
 * makes it appear to pulsate,
 */
#define STARPOINTS 5
#define STARRADIUS 40
#define PI 3.1416
void DrawStar (int x, int y, int color) {

	int sp;
	double angle;
	int x1, y1;

	sp = STARPOINTS + (rand () %3);
	for (angle = 0.0; angle < 2 * PI; angle = angle + (2*PI)/sp) {
		x1 = (int) (cos (angle) * STARRADIUS);
		y1 = (int) (sin (angle) * STARRADIUS);
		GP142_lineXY (color, x+x1, y+y1, x-x1, y-y1, 1);

	}
}
/* 
 * Draw an arrow on the card.
 * This arrow is pointing to the right.
 */
void DrawArrow (int x, int y, int color) {

	int x1, y1;
	
	x1 = 15 + (rand () %4);
	y1 = x1;

		GP142_lineXY (color, x, y, x-x1, y+y1, 3);
		GP142_lineXY (color, x, y, x-x1, y-y1, 3);
		GP142_lineXY (color, x, y, x-30-x1, y, 3);
}
/* 
 * Draw an arrow on the card.
 * This arrow is pointing to the left.
 */
void DrawArrow_2 (int x, int y, int color) {

	int x1, y1;
	
	x1 = 15 + (rand () %4);
	y1 = x1;

	GP142_lineXY (color, x, y, x+x1, y+y1, 3);
	GP142_lineXY (color, x, y, x+x1, y-y1, 3);
	GP142_lineXY (color, x, y, x+30+x1, y, 3);
}
/*
 * Draw a birthday cake on the card.
 * The cake can change colors, but the flames are red.
 * The flicker is made by setting the points on 
 * the triangle that forms the flame to be random numbers
 * within a small range.
 */
void DrawCake (int x, int y, int color) {


	GP142_rectangleXY (color, x, y, x + 50, y + 30, FILL);
	GP142_lineXY (PURPLE, x + 10, y + 31, x + 10, y + 40, 2);
	GP142_lineXY (PINK, x + 20, y + 31, x + 20, y + 40, 2);
	GP142_lineXY (GREEN, x + 30, y + 31, x + 30, y + 40, 2);
	GP142_lineXY (YELLOW, x + 40, y + 31, x + 40, y + 40, 2);
	GP142_triangleXY (ORANGE, x + 9, y + 43,
		x + 10, y + 45 + rand () %2, x + 11, y + 43, FILL);
	GP142_triangleXY (ORANGE, x + 19, y + 43,
		x + 20, y + 45 + rand () %2, x + 21, y + 43, FILL);
	GP142_triangleXY (ORANGE, x + 29, y + 43,
		x + 30, y + 45 + rand () %2, x + 31, y + 43, FILL);
	GP142_triangleXY (ORANGE, x + 39, y + 43,
		x + 40, y + 45 + rand () %2, x + 41, y + 43, FILL);
}
/* Draw a bunch of three balloons*/

void DrawBalloons (int x, int y, int color) {

	int x1 = x + 2 + rand ()%3,
		y1 = y - 80 + rand ()%3;

	GP142_circleXY(color, x - 30, y - 10, 18);
	GP142_circleXY(color + 1, x, y, 20);
	GP142_circleXY(color + 2, x + 30, y - 13, 19);
	GP142_lineXY(color + 1, x - 30, y - 28, x1 + 8, y1 - 8, 2);
	GP142_lineXY(color + 2, x, y - 20, x1, y1 - 8, 2);
	GP142_lineXY(color + 3, x + 30, y - 30, x1 - 8, y1 - 8, 2);
	GP142_ovalXY(color + 1, x1 - 6, y1, x1, y1 + 4, FILL);
	GP142_ovalXY(color + 1, x1, y1, x1 + 6, y1 + 4, FILL);
}
/* Draw a christmas tree.
 * The lights are formed by small circles with random colors
 */
void DrawTree (int x, int y, int color) {

	int n = rand ()%24;

	GP142_triangleXY (GREEN, x, y, x - 20, y - 34, x + 20, y - 34, FILL);
	GP142_triangleXY (GREEN, x, y + 16, x - 16, y - 16, x + 16, y - 16, FILL);
	GP142_rectangleXY (BROWN, x - 4, y - 41, x + 4, y - 35, FILL);
	GP142_circleXY (color + n, x - 2, y, 2);
	GP142_circleXY (color + 5 + n, x - 4, y - 9, 2);
	GP142_circleXY (color + 3 + n, x - 7, y - 28, 2);
	GP142_circleXY (color + 12 + n, x + 3, y - 30, 2);
	GP142_circleXY (color + 19 + n, x + 6, y - 3, 2);
	GP142_circleXY (color + 7 + n, x + 3, y - 18, 2);
	GP142_circleXY (color + 6 + n, x + 10, y - 29, 2);
	GP142_circleXY (WHITE, x, y + 19, 3);
	GP142_triangleXY (WHITE, x, y + 19, x - 5, y + 10, x + 5, y + 10, FILL);
	GP142_circleXY (WHITE, x, y + 24, 2);
}
/* Add a message to the card */
void DrawMessage(int shape_type, int color, int x, int y)
{

	int font_size = 15 + rand ()%3;

	switch ((shape_type - FIRST_MESSAGE_NUM) % MAX_MESSAGES)
	{
	case HAPPY_BIRTHDAY:
		GP142_printfXY(color, x, y, font_size + 1, "Happy Birthday!");
		break;
	case I_LOVE_YOU:
		GP142_printfXY(color, x, y, font_size + 1, "I love you");
		break;
	case BE_MINE:
		GP142_printfXY(color, x, y, font_size + 1, "Be mine...");
		break;
	case CONGRATULATIONS:
		GP142_printfXY(color, x, y, font_size + 5, "Congratulations!!");
		break;
	case MERRY_CHRISTMAS:
		GP142_printfXY(color, x, y, font_size + 1, "Merry Christmas!");
		break;
	case THANK_YOU:
		GP142_printfXY(color, x, y, font_size + 1, "Thank you");
		break;
	default:
		/* Unknown shape selected */
		break;
		} /*end switch*/



}


/*
 * Mouse clicks occur in one of three regions of interest: on buttons, 
 * on the card, or anywhere else.
 */
int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
  if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH) &&
      (mouse_y < GP142_YMAX) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT)))
    return CLICK_BUTTON;
  else if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
	  && mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
    return CLICK_CARD;
  else
    return CLICK_OUTSIDE;

}


/*
 * Decide where a mouse click occured, and react appropriately.
 */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[], 
					  int object_num[], int object_color[], 
					  int object_x[], int object_y[], int *current_object,
					  int *last_button)
{

	int mouse_click;
	mouse_click = ClassifyMouseClick(mouse_x, mouse_y);
			
	if (mouse_click == CLICK_BUTTON) {
				
		/* A button has been clicked.  Call a function to handle it*/
		/* most of the variables passed as args will NOT change*/
		HandleButton(mouse_x, mouse_y, background_color, object_color, status, 
					 *object_count, object_type, object_num, *current_object, last_button);

	
	}

	else if (mouse_click == CLICK_CARD) {
		/* Add a shape, if there isn't one already */
		
		switch (*status) {
		case STATUS_ADD_SHAPE:
			AddShape(mouse_x, mouse_y, status, object_count, 
				object_type, object_num, object_color, object_x, object_y, 
				current_object);
			break;
		case STATUS_ADD_TEXT:
			AddText(mouse_x, mouse_y, status, object_count, 
				object_type, object_num,
				object_color, object_x, object_y, current_object);
			break;
		case STATUS_SELECTED:
			MoveObject(mouse_x, mouse_y, current_object, object_x, object_y);
			break;
		case STATUS_READY_TO_SELECT:
			*current_object = Get_Object(mouse_x, mouse_y, object_x, object_y,
				object_count);
			*status = STATUS_SELECTED;
			break;
		}


	}
	/* Ignore any other mouse clicks */
}

/* 
 * Figure out which button was clicked, if any, and do the right 
 * thing.
 */
void HandleButton(int x, int y, int *background_color, int object_color[], 
				  int *status, int object_count, int object_type[], int object_num[],
				  int current_object, int *last_button)
{
	int buttonNum = 0;
	
	/* If we're outside of this area, we could not have clicked on
	* a button.
	*/
	if (x > -GP142_XMAX + BUTTON_WIDTH)  
		return ;	/*That's an error situation*/
	
					/* We know the mouse clicked in the column of buttons. Calculate
					* which one.
	*/
	buttonNum = (GP142_YMAX - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		/* Change the background color */
		*background_color = (*background_color + 1) % MAX_COLORS;
		*last_button = 0;
		break;
	case 1:
		/* Change the type of the currently selected object */
		if (object_type[current_object] == OBJECT_SHAPE){
			object_num[current_object] = 
				(object_num[current_object] + 1) % MAX_SHAPES;
		}
		if (object_type[current_object] == OBJECT_MESSAGE){
			object_num[current_object] = 
				(object_num[current_object] + 1) % MAX_MESSAGES;
		}
		*last_button = 1;
		break;
	case 2:
		/* Change the color of the selected object*/
		if (object_type[current_object] == OBJECT_SHAPE){
			object_color[current_object] = 
				(object_color[current_object] + 1) % MAX_COLORS;
		}
		if (object_type[current_object] == OBJECT_MESSAGE){
			object_color[current_object] = 
				(object_color[current_object] + 1) % MAX_COLORS;
		}
		*last_button = 2;
		break;
	case 3:
		/* Add another shape*/
		*status = STATUS_ADD_SHAPE;
		*last_button = 3;
		break;
	case 4:
		/*add some text*/
		*status = STATUS_ADD_TEXT;
		*last_button = 4;
		break;
	case 5:
		/* select a different object*/
		*status = STATUS_READY_TO_SELECT;
		*last_button = 5;
		break;
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
		break;
	}
}


/* If there's room for another shape, add it to the card. */
void AddShape (int x, int y, int *status, int *object_count, 
			   int object_type[], int object_num[], int object_color[], 
			   int object_x[], int object_y[], int *current_object)
{
	if (*object_count < MAX_OBJECTS) {
		*status = STATUS_SELECTED;
		object_type[*object_count] = OBJECT_SHAPE;
		object_num[*object_count] = SHAPE_HEART;
		object_color[*object_count] = BLACK;
		object_x[*object_count] = x;
		object_y[*object_count] = y;
			*current_object = *object_count;
		*object_count = *object_count + 1;
	
	}
	
}
/* If there's room for another message, add it to the card.  */
void AddText (int x, int y, int *status, int *object_count, int object_type[],
			 int object_num[], int object_color[], int object_x[],
			 int object_y[], int *current_object)
{
	if (*object_count < MAX_OBJECTS) {
		*status = STATUS_SELECTED;
		object_type[*object_count] = OBJECT_MESSAGE;
		object_num[*object_count] = HAPPY_BIRTHDAY;
		object_color[*object_count] = BLACK;
		object_x[*object_count] = x;
		object_y[*object_count] = y;
		*current_object = *object_count;
		*object_count = *object_count + 1;
		

	}
}
/* Update x- and y- coordinates of an object when it is moved*/
void MoveObject (int x, int y, int *current_object, int object_x[], 
				 int object_y[]) {
	object_x[*current_object] = x;
	object_y[*current_object] = y;

}

int Get_Object (int x, int y, int object_x[], int object_y[], 
				int *object_count) {
	int i, closest;
	double distance = 10.0,
		   n = 400.0;

	for (i = 0; i < *object_count; i++) {
		distance = sqrt((x - object_x[i]) * (x - object_x[i]) +
				(y - object_y[i]) * (y - object_y[i]));
		if (distance < n){
			n = distance;
			closest = i;
		}
	}

	return (closest);
}




